#from .internal import *
from .message import *
from .embed import *
from .guild import *
from .ready import *
from .file import *
from .http import *
from .main import *
from .role import *
from .user import *
from . import utils

__version__ = "1.1.1"

#what the hell is "imports is un-sorted/formatted"!?!?!?!?
